package sun.net.www.protocol.jar;

import sun.net.www.ParseUtil;
import java.net.MalformedURLException;
import java.io.IOException;
import java.net.URLConnection;
import java.net.URL;
import java.net.URLStreamHandler;

public class Handler extends URLStreamHandler
{
    private static final String separator = "!/";
    
    public Handler() {
    }
    
    @Override
    protected URLConnection openConnection(URL url) throws IOException {
        return new JarURLConnection(url, this);
    }
    
    private static int indexOfBangSlash(String s) {
        int fromIndex = s.length();
        while ((fromIndex = s.lastIndexOf(33, fromIndex)) != -1) {
            if (fromIndex != s.length() - 1 && s.charAt(fromIndex + 1) == '/') {
                return fromIndex + 1;
            }
            --fromIndex;
        }
        return -1;
    }
    
    @Override
    protected boolean sameFile(URL url, URL url2) {
        if (!url.getProtocol().equals("jar") || !url2.getProtocol().equals("jar")) {
            return false;
        }
        String file = url.getFile();
        String file2 = url2.getFile();
        int index = file.indexOf("!/");
        int index2 = file2.indexOf("!/");
        if (index == -1 || index2 == -1) {
            return super.sameFile(url, url2);
        }
        if (!file.substring(index + 2).equals(file2.substring(index2 + 2))) {
            return false;
        }
        URL u1;
        URL u2;
        try {
            u1 = new URL(file.substring(0, index));
            u2 = new URL(file2.substring(0, index2));
        }
        catch (MalformedURLException ex) {
            return super.sameFile(url, url2);
        }
        return super.sameFile(u1, u2);
    }
    
    @Override
    protected int hashCode(URL url) {
        int n = 0;
        String protocol = url.getProtocol();
        if (protocol != null) {
            n += protocol.hashCode();
        }
        String file = url.getFile();
        int index = file.indexOf("!/");
        if (index == -1) {
            return n + file.hashCode();
        }
        String substring = file.substring(0, index);
        try {
            n += new URL(substring).hashCode();
        }
        catch (MalformedURLException ex) {
            n += substring.hashCode();
        }
        return n + file.substring(index + 2).hashCode();
    }
    
    public String checkNestedProtocol(String s) {
        if (s.regionMatches(true, 0, "jar:", 0, 4)) {
            return "Nested JAR URLs are not supported";
        }
        return null;
    }
    
    @Override
    protected void parseURL(URL u, String substring, int beginIndex, int n) {
        String file = null;
        String substring2 = null;
        int index = substring.indexOf(35, n);
        int n2 = index == beginIndex;
        if (index > -1) {
            substring2 = substring.substring(index + 1, substring.length());
            if (!(n2 == 0)) {
                file = u.getFile();
            }
        }
        boolean equalsIgnoreCase = false;
        if (substring.length() >= 4) {
            equalsIgnoreCase = substring.substring(0, 4).equalsIgnoreCase("jar:");
        }
        substring = substring.substring(beginIndex, n);
        String checkNestedProtocol = this.checkNestedProtocol(substring);
        if (checkNestedProtocol != null) {
            throw new NullPointerException(checkNestedProtocol);
        }
        if (!(equalsIgnoreCase == 0)) {
            file = this.parseAbsoluteSpec(substring);
        }
        else if (n2 == 0) {
            String contextSpec = this.parseContextSpec(u, substring);
            int indexOfBangSlash = indexOfBangSlash(contextSpec);
            file = contextSpec.substring(0, indexOfBangSlash) + new ParseUtil().canonizeString(contextSpec.substring(indexOfBangSlash));
        }
        this.setURL(u, "jar", "", -1, file, substring2);
        return;
    }
    
    private String parseAbsoluteSpec(String str) {
        int indexOfBangSlash;
        if ((indexOfBangSlash = indexOfBangSlash(str)) == -1) {
            throw new NullPointerException("no !/ in spec");
        }
        try {
            URL url = new URL(str.substring(0, indexOfBangSlash - 1));
        }
        catch (MalformedURLException obj) {
            throw new NullPointerException("invalid url: " + str + " (" + obj + ")");
        }
        return str;
    }
    
    private String parseContextSpec(URL url, String str) {
        String str2 = url.getFile();
        if (str.startsWith("/")) {
            int indexOfBangSlash = indexOfBangSlash(str2);
            if (indexOfBangSlash == -1) {
                throw new NullPointerException("malformed context url:" + url + ": no !/");
            }
            str2 = str2.substring(0, indexOfBangSlash);
        }
        if (!str2.endsWith("/") && !str.startsWith("/")) {
            int lastIndex = str2.lastIndexOf(47);
            if (lastIndex == -1) {
                throw new NullPointerException("malformed context url:" + url);
            }
            str2 = str2.substring(0, lastIndex + 1);
        }
        return str2 + str;
    }
}
